<?php
namespace Teknisa\EST\Modules\Produto\Factories;

use Doctrine\ORM\EntityManager;

class Produto {
    
    protected $entityManager;
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }

    public function getArvprodByCdproduto($params) {
        $sql = 'SELECT CDARVPROD 
                FROM PRODUTO
                WHERE CDPRODUTO = :CDPRODUTO
                  AND NRORG = :NRORG';

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result['CDARVPROD'];
    }

    public function getChildren($params) {
        $sql = "SELECT CDNVPRODUTO,
                  CDPRODNIVANTE,
                  CDARVPROD,
                  CDPRODUTO,
                CASE
                    WHEN REGEXP_LIKE(SUBSTR(CDARVPROD, 1, 1), '([A-Z])')
                    THEN '99'
                    ELSE SUBSTR(CDARVPROD, 1, 1)
                END ALPHANUMERIC_ORDER
                FROM PRODUTO
                WHERE (:CDPRODUTO IS NULL
                  AND CDPRODNIVANTE IS NULL)
                OR (:CDPRODUTO    IS NOT NULL
                  AND CDPRODNIVANTE = :CDPRODUTO)
                AND NRORG          = :NRORG
                ORDER BY ALPHANUMERIC_ORDER DESC, 
                  CDARVPROD DESC";

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }

    public function getMaxLevel($params) {
        $sql = "SELECT MAX(CDNVPRODUTO) AS MAXLEVEL
                FROM PRODUTO P 
                JOIN PRODFILI PF 
                  ON PF.CDPRODUTO = P.CDPRODUTO 
                  AND PF.NRORG = P.NRORG 
                WHERE (PF.IDPRODATIV = :IDPRODATIV 
                       OR NVL(:IDPRODATIV, 'T') = 'T') 
                AND P.NRORG = :NRORG";

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result['MAXLEVEL'];
    }

    public function getProductByArvprod($params) {
        $sql = "SELECT * 
                FROM PRODUTO 
                WHERE CDARVPROD = :CDARVPROD
                AND NRORG = :NRORG";

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result;
    }

    public function getProductByCode($params) {
        $sql = "SELECT * 
                FROM PRODUTO 
                WHERE CDPRODUTO = :CDPRODUTO
                AND NRORG = :NRORG";

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result;
    }

    public function getProductMask($params) {
        $sql = "SELECT CDPICTPROD 
                FROM PARAMGERAL 
                WHERE NRORG = :NRORG";

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result['CDPICTPROD'];
    }

    public function getProductType($params) {
        $sql = "SELECT CASE WHEN CDPRODUTO = CDPRODESTO 
                         THEN 'E' 
                         ELSE 'C' 
                       END AS IDESTOCOMP 
                FROM PRODUTO
                WHERE CDPRODUTO = :CDPRODUTO
                AND NRORG = :NRORG";

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        $result = $query->fetch();
        return $result['IDESTOCOMP'];
    }

    public function getTpTreeProduct($params) {
        $sql = "SELECT NRTPARVPROD,
                       NRARVOREPROD,
                       CDPRODUTO,
                       NRORG,
                       DTINCLUSAO,
                       CDOPERINCLUSAO,
                       DTULTATU,
                       NRORGINCLUSAO,
                       NRORGULTATU,
                       CDOPERULTATU,
                       IDATIVO
                FROM
                    PRODARVOREPROD 
                WHERE CDPRODUTO = :CDPRODUTO 
                AND NRORG = :NRORG";

        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();
    }
    
    public function getSegMercFilial($cdFilial) {
        $sql = "SELECT
                   F.CDFILIAL, F.CDSEGMER
                FROM
                   FILIAL F
                WHERE
                   F.CDFILIAL = :P_CDFILIAL";
                   
        $params = array("P_CDFILIAL" => $cdFilial);           
                   
        $query = $this->connection->prepare($sql);
        $query->execute($params);
        return $query->fetchAll();           
    }   
}
